/*******************************************************************************
 * time.H - Time class
 *
 * $Id: time.H,v 2.0 2001/05/23 11:49:10 mscheid Exp $
 *
 * Author: Matthias Scheidegger <mscheid@iam.unibe.ch>
 *
 * Copyright 2001, Matthias Scheidegger
 *
 ******************************************************************************/

#ifndef __TIME_H
#define __TIME_H

#include "types.H"
#include "exception.H"
#include <time.h>
#include <sys/time.h>
#include <unistd.h>


class Time
{
    double t;
    
  public:
    Time()                  	   { set(); }                  // default, set current
    Time(double seconds)    	   : t(seconds) {}
    Time(__u64 ntpt)      	   { set_to(ntpt); }
    Time(__u32 tntpt)      	   { set_to(tntpt); }
    Time(const struct timeval *tv) { set_to(tv); }
    Time(const struct timespec *ts){ set_to(ts); }
    Time(const Time &o)            { t = o.t; }                // copy constructor
    Time& operator=(const Time &o) { t = o.t; return *this; }      // assignments
    Time& operator=(double seconds){ t = seconds; return *this; }

    // writing methods
    void set();                           // set to current time
    void set_to(double seconds);	  // set to number of seconds
    void set_to(__u64 ntpt);     	  // set time from NTP timestamp
    void set_to(__u32 tntpt);     	  // set time from truncated NTP timestamp
    void set_to(const struct timeval*);   // set time from timeval
    void set_to(const struct timespec*);  // set time from timespec

    // reading methods
    operator double() const { return t; }  	        // cast Time to double
    double get_double() const	   { return t; }        // get Time as double
    __u64 get_ntp() const;                      	// get Time as NTP timestamp
    __u32 get_trunc_ntp() const;                	// get Time as truncated NTP timestamp
    struct timeval* get_timeval(struct timeval*) const; // fill timeval struct
    struct timespec* get_timespec(struct timespec*) const; // fill timespec struct

    // comparison
    bool operator==(const Time &o) { return t == o.t; }
    bool operator!=(const Time &o) { return t != o.t; }
    bool operator<(const Time &o)  { return t < o.t;  }
    bool operator<=(const Time &o) { return t <= o.t; }
    bool operator>(const Time &o)  { return t > o.t;  }
    bool operator>=(const Time &o) { return t >= o.t; }

    // arithmetic ops
    Time operator+(const Time &o)   { return Time(t + o.t); }
    Time operator-(const Time &o)   { return Time(t - o.t); }
    Time operator*(double scalar)   { return Time(t * scalar); }
    Time operator/(double scalar)   { return Time(t / scalar); }
    Time& operator+=(const Time &o) { t += o.t; }
    Time& operator-=(const Time &o) { t -= o.t; }
    Time& operator*=(double scalar) { t *= scalar; }
    Time& operator/=(double scalar) { t /= scalar; }
};



#endif
