/*******************************************************************************
 * udprcv.C - UDP receiver, output bandwidth received on a specific port
 *
 * $Id: udprcv.C,v 2.0 2001/05/23 11:49:10 mscheid Exp $
 *
 * Author: Matthias Scheidegger <mscheid@iam.unibe.ch>
 *
 * Copyright 2001, Matthias Scheidegger
 *
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "time.H"
#include "netconsts.H"


//-- Usage ---------------------------------------------------------------------

static char *exe_name = NULL;     // name the executable (as supplied in argv)

void usage()
{
    fprintf(stderr, "Usage: %s <port>\n",
	    exe_name);
    exit(2);
}


//-- Main ----------------------------------------------------------------------

void receive(int port);
void pexit(const char *f);


int main(int argc, char **argv)
{
    int port=0;
    
    // parse args
    exe_name = argv[0];
    argc--; argv++;
    
    if (argc != 1) usage();
    port = atoi(argv[0]);
    if (port == 0) usage();
    
    receive(port);
    return 0;
}


void receive(int port)
{
    int s;
    sockaddr_in addr;
    Time start(0.0);
    Time lastprint(0.0);
    Time recvtime(0.0);
    int nread;
    unsigned long lately_read;
    const int header_len = IP_HEADER_LEN + UDP_HEADER_LEN;
    const double min_print_ival = 0.1;
    char buf[0x10000];

    // make stdout line buffered
    setvbuf(stdout, NULL, _IOLBF, 0);
    
    // create address
    bzero(&addr, sizeof(struct sockaddr_in));
    addr.sin_family = AF_INET;
    addr.sin_port = htons((short)port);
    addr.sin_addr.s_addr = INADDR_ANY;
    
    if ( (s = socket(AF_INET, SOCK_DGRAM, 0)) == -1) pexit("socket");
    if (bind(s, (struct sockaddr*)&addr, sizeof(struct sockaddr_in)) == -1) pexit("bind");
    
    lately_read = 0;
    while (42) {
	nread = recv(s, buf, 0x10000, 0);
	if ((double)start == 0.0) {  // first packet
	    start.set();
	    recvtime = lastprint = start;
	} else {
	    recvtime.set();	    
	}
	lately_read += nread + header_len;
	if ( min_print_ival <= (recvtime-lastprint)) {
	    printf("%g %g\n", (double)(recvtime - start), (double)lately_read/(recvtime-lastprint));
	    lately_read = 0;
	    lastprint = recvtime;
	}
    }
}


void pexit(const char *f)
{
    perror(f);
    exit(2);
}
